import { Effect, Reducer, Subscription } from 'umi';
import { ActivitiesType, CurrentUser, NoticeType, RadarDataType } from './data.d';
import {
  fakeChartData, queryActivities, queryCurrent, queryProjectNotice, getProductList, getProductInfoGateway, getProductInfoOneSwitch,
  getProductInfoThreeSwitch, getProductInfoSensorBody, getProductInfoLight, getProductInfoSocket, getProductInfoSuperSocket, getProductInfoTemperature,
  getProductInfoLock, execApi, getProductAllInfo
} from './service';


import emitter from '@/utils/events';

export interface ModalState {
  currentUser?: CurrentUser;
  projectNotice: NoticeType[];
  activities: ActivitiesType[];
  radarData: RadarDataType[];
}

export interface ModelType {
  namespace: string;
  state: ModalState;
  reducers: {
    save: Reducer<ModalState>;
    clear: Reducer<ModalState>;
    getProductInfoGateway: Reducer<ModalState>;
    getProductInfoOneSwitch: Reducer<ModalState>;
    getProductInfoThreeSwitch: Reducer<ModalState>;
    getProductInfoSensorBody: Reducer<ModalState>;
    getProductInfoLight: Reducer<ModalState>;
    getProductInfoSocket: Reducer<ModalState>;
    getProductInfoSuperSocket: Reducer<ModalState>;
    getProductInfoTemperature: Reducer<ModalState>;
    getProductInfoLock: Reducer<ModalState>;
    exec: Reducer<ModalState>;
    update: Reducer<ModalState>;
  };
  effects: {
    init: Effect;
    getProductList: Effect;
    getProductAllInfo: Effect;
    getProductInfoGateway: Effect;
    getProductInfoOneSwitch: Effect;
    getProductInfoThreeSwitch: Effect;
    getProductInfoSensorBody: Effect;
    getProductInfoLight: Effect;
    getProductInfoSocket: Effect;
    getProductInfoLock: Effect;
    getProductInfoTemperature: Effect;
    getProductInfoSuperSocket: Effect;
    fetchUserCurrent: Effect;
    fetchProjectNotice: Effect;
    fetchActivitiesList: Effect;
    fetchChart: Effect;
    exec: Effect;
  };
  subscriptions: { setup: Subscription };
}

const Model: ModelType = {
  namespace: 'dashboardAndworkplace',
  state: {
    currentUser: undefined,
    projectNotice: [],
    activities: [],
    radarData: [],
  },
  effects: {
    *init(_, { put }) {
      yield put({ type: 'fetchUserCurrent' });
      yield put({ type: 'fetchProjectNotice' });
      yield put({ type: 'fetchActivitiesList' });
      yield put({ type: 'fetchChart' });
    },
    *fetchUserCurrent(_, { call, put }) {
      const response = yield call(queryCurrent);
      yield put({
        type: 'save',
        payload: {
          currentUser: response,
        },
      });
    },
    *fetchProjectNotice(_, { call, put }) {
      const response = yield call(queryProjectNotice);
      yield put({
        type: 'save',
        payload: {
          projectNotice: Array.isArray(response) ? response : [],
        },
      });
    },
    *fetchActivitiesList(_, { call, put }) {
      const response = yield call(queryActivities);
      yield put({
        type: 'save',
        payload: {
          activities: Array.isArray(response) ? response : [],
        },
      });
    },

    //开关设备
    *exec({ payload, callback }, { call, put }) {
      const response = yield call(execApi, payload);
    },

    //获取设备列表
    *getProductList({ payload, callback }, { call, put }) {
      const response = yield call(getProductList, payload);
      if (callback) callback(response);
    },

    //获取所有设备

    *getProductAllInfo({ payload, callback }, { call, put }) {
      const response = yield call(getProductAllInfo, payload);
      yield put({
        type: 'save',
        payload: {
          productAllList: response.model.list,
        },
      });
    },
    //获取单个设备信息(网关)
    *getProductInfoGateway({ payload, callback }, { call, put }) {
      const response = yield call(getProductInfoGateway, payload);
      let newAry: any = [];
      response.devices.map((item: any) => {
        newAry.push({
          key: item.id,
          name: item.displayName,
          className: item.node != null ? item.node.name : "",
          status: item.isOnline,
          version: item.data[1].value,
          number: item.number,
        })
      })

      yield put({
        type: 'save',
        payload: {
          productGateway: newAry,
        },
      });

      if (callback) callback(newAry);
    },

    //获取单个设备信息(一路开关)
    *getProductInfoOneSwitch({ payload, callback }, { call, put }) {
      const response = yield call(getProductInfoOneSwitch, payload);
      let newAry: any = [];

      const getState = (ary: any) => {
        let state = '';

        ary.map((item: any) => {
          if (item.key == 'State') {
            state = item.value
          }
        })

        return state
      }

      response.devices.map((item: any) => {
        newAry.push({
          key: item.id,
          name: item.displayName,
          className: item.node != null ? item.node.name : "",
          status: getState(item.data),
          number: item.number,
          isOnline: item.isOnline,
        })
      })
      yield put({
        type: 'save',
        payload: {
          productOneSwitch: newAry,
        },
      });
    },


    //获取单个设备信息(三路开关)
    *getProductInfoThreeSwitch({ payload, callback }, { call, put }) {
      const response = yield call(getProductInfoThreeSwitch, payload);
      let newAry: any = [];
      const getState = (ary: any) => {
        let state = '';

        ary.map((item: any) => {
          if (item.key == 'State') {
            state = item.value
          }
        })

        return state
      }
      const getState1 = (ary: any) => {
        let state = '';

        ary.map((item: any) => {
          if (item.key == 'L1State') {
            state = item.value
          }
        })

        return state
      }
      const getState2 = (ary: any) => {
        let state = '';

        ary.map((item: any) => {
          if (item.key == 'L2State') {
            state = item.value
          }
        })

        return state
      }
      const getState3 = (ary: any) => {
        let state = '';

        ary.map((item: any) => {
          if (item.key == 'L3State') {
            state = item.value
          }
        })

        return state
      }

      response.devices.map((item: any) => {
        newAry.push({
          key: item.id,
          name: item.displayName,
          className: item.node != null ? item.node.name : "",
          status1: getState1(item.data),
          status2: getState2(item.data),
          status3: getState3(item.data),
          status: getState(item.data),
          isOnline: item.isOnline,
          number: item.number,
        })
      })

      yield put({
        type: 'save',
        payload: {
          productThreeSwitch: newAry,
        },
      });
    },


    //获取单个设备信息(人体感应器)
    *getProductInfoSensorBody({ payload, callback }, { call, put }) {
      const response = yield call(getProductInfoSensorBody, payload);
      let newAry: any = [];

      response.devices.map((item: any) => {
        newAry.push({
          key: item.id,
          name: item.displayName,
          className: item.node != null ? item.node.name : "",
          status: item.data[0].value,
          gateStatus: item.isOnline,
          number: item.number,
        })
      })

      yield put({
        type: 'save',
        payload: {
          productSensorBody: newAry,
        },
      });
    },

    //获取单个设备信息(光强检测器)
    *getProductInfoLight({ payload, callback }, { call, put }) {
      const response = yield call(getProductInfoLight, payload);

      let newAry: any = [];
      const getState = (ary: any) => {
        let state = '';

        ary.map((item: any) => {
          if (item.key == 'Light') {
            state = `${item.value} ${item.unit}`
          }
        })

        return state
      }
      response.devices.map((item: any) => {
        newAry.push({
          key: item.id,
          name: item.displayName,
          className: item.node != null ? item.node.name : "",
          status: item.isOnline,
          lightInfo: getState(item.data),
          number: item.number,
        })
      })

      yield put({
        type: 'save',
        payload: {
          productLight: newAry,
        },
      });
    },

    //获取单个设备信息(插座)
    *getProductInfoSocket({ payload, callback }, { call, put }) {
      const response = yield call(getProductInfoSocket, payload);
      let newAry: any = [];
      const getState = (ary: any) => {
        let state = '';

        ary.map((item: any) => {
          if (item.key == 'State') {
            state = item.value
          }
        })

        return state
      }
      response.devices.map((item: any) => {
        newAry.push({
          key: item.id,
          name: item.displayName,
          className: item.node != null ? item.node.name : "",
          status: item.isOnline,
          severStatus: getState(item.data),
          number: item.number,
        })
      })

      yield put({
        type: 'save',
        payload: {
          productSocket: newAry,
        },
      });
    },

    //获取单个设备信息(智能插座)
    *getProductInfoSuperSocket({ payload, callback }, { call, put }) {
      const response = yield call(getProductInfoSuperSocket, payload);
      let newAry: any = [];

      const getState = (ary: any) => {
        let state = '';

        ary.map((item: any) => {
          if (item.key == 'State') {
            state = item.value
          }
        })

        return state
      }
      response.devices.map((item: any) => {
        newAry.push({
          key: item.id,
          name: item.displayName,
          className: item.node != null ? item.node.name : "",
          status: item.isOnline,
          severStatus: getState(item.data),
          number: item.number,
        })
      })

      yield put({
        type: 'save',
        payload: {
          productSuperSocket: newAry,
        },
      });
    },

    //获取单个设备信息(门锁)
    *getProductInfoLock({ payload, callback }, { call, put }) {
      const response = yield call(getProductInfoLock, payload);
      let newAry: any = [];

      const getState = (ary: any) => {
        let state = '';

        ary.map((item: any) => {
          if (item.key == 'State') {
            state = item.value
          }
        })

        return state
      }
      response.devices.map((item: any) => {
        newAry.push({
          key: item.id,
          name: item.displayName,
          className: item.node != null ? item.node.name : "",
          status: item.isOnline,
          severStatus: getState(item.data),
          number: item.number,
        })
      })

      yield put({
        type: 'save',
        payload: {
          productLock: newAry,
        },
      });
      if (callback) callback(newAry);
    },

    //获取单个设备信息(温湿度传感器)
    *getProductInfoTemperature({ payload, callback }, { call, put }) {
      const response = yield call(getProductInfoTemperature, payload);
      let newAry: any = [];

      const getTemState = (ary: any) => {
        let state = '';

        ary.map((item: any) => {
          if (item.key == 'Temperature') {
            state = `${item.value} ${item.unit}`
          }
        })

        return state
      }

      const getHuState = (ary: any) => {
        let state = '';

        ary.map((item: any) => {
          if (item.key == 'Humidity') {
            state = `${item.value} ${item.unit}`
          }
        })

        return state
      }

      response.devices.map((item: any) => {
        newAry.push({
          key: item.id,
          name: item.displayName,
          className: item.node != null ? item.node.name : "",
          status: item.isOnline,
          humidity: getHuState(item.data),
          temperature: getTemState(item.data),
          number: item.number,
        })
      })

      yield put({
        type: 'save',
        payload: {
          productTemperature: newAry,
        },
      });
    },



    *fetchChart(_, { call, put }) {
      const { radarData } = yield call(fakeChartData);
      yield put({
        type: 'save',
        payload: {
          radarData,
        },
      });
    },
  },
  reducers: {
    save(state, { payload }) {
      return {
        ...state,
        ...payload,
      };
    },
    clear() {
      return {
        currentUser: undefined,
        projectNotice: [],
        activities: [],
        radarData: [],
      };
    },

    update(state, action) {

      console.log(action, 'action')
      console.log(state, 'state')
      if (action.payload.model.device.name == "三路开关") {
        if (state.productThreeSwitch && state.productThreeSwitch.length > 0) {
          state.productThreeSwitch.map((item: any) => {
            if (item.number == action.payload.model.device.number) {
              item.isOnline = action.payload.model.device.isOnline
              if (action.payload.model.key == "L1State") {
                item.status1 = action.payload.model.value
              }
              if (action.payload.model.key == "L2State") {
                item.status2 = action.payload.model.value
              }

              if (action.payload.model.key == "L3State") {
                item.status3 = action.payload.model.value
              }

              if (action.payload.model.key == "State") {
                item.status = action.payload.model.value
              }

            }
          })
        }
      }

      if (action.payload.model.device.name == "一路开关") {
        if (state.productOneSwitch && state.productOneSwitch.length > 0) {
          state.productOneSwitch.map((item: any) => {
            if (item.number == action.payload.model.device.number) {
              item.isOnline = action.payload.model.device.isOnline
              item.status = action.payload.model.value
            }
          })
        }
      }

      if (action.payload.model.device.name == "插座") {
        if (state.productSocket && state.productSocket.length > 0) {
          state.productSocket.map((item: any) => {
            if (item.number == action.payload.model.device.number) {
              item.severStatus = action.payload.model.value
              item.status = action.payload.model.device.isOnline
            }
          })
        }
      }

      if (action.payload.model.device.name == "智能插座") {
        if (state.productSuperSocket && state.productSuperSocket.length > 0) {
          state.productSuperSocket.map((item: any) => {
            if (item.number == action.payload.model.device.number) {

              item.status = action.payload.model.device.isOnline
              if (action.payload.model.key == "State") {
                item.severStatus = action.payload.model.value
              }

            }
          })
        }
      }

      if (action.payload.model.device.name == "门锁") {
        if (state.productLock && state.productLock.length > 0) {
          state.productLock.map((item: any) => {
            if (item.number == action.payload.model.device.number) {
              item.status = action.payload.model.device.isOnline
              item.severStatus = action.payload.model.value
            }
          })
        }
      }

      if (action.payload.model.device.name == "光强检测器") {
        if (state.productLight && state.productLight.length > 0) {
          state.productLight.map((item: any) => {
            if (item.number == action.payload.model.device.number) {
              item.lightInfo = `${action.payload.model.value} ${action.payload.model.unit}`
              item.status = action.payload.model.device.isOnline
            }
          })
        }
      }

      if (action.payload.model.device.name == "人体感应器") {
        if (state.productSensorBody && state.productSensorBody.length > 0) {
          state.productSensorBody.map((item: any) => {
            if (item.number == action.payload.model.device.number) {
              item.status = action.payload.model.value
              item.gateStatus = action.payload.model.device.isOnline


            }
          })
        }
      }


      if (action.payload.model.device.name == "温湿度传感器") {
        if (state.productTemperature && state.productTemperature.length > 0) {
          state.productTemperature.map((item: any) => {
            if (item.number == action.payload.model.device.number) {
              item.status = action.payload.model.device.isOnline
              if (action.payload.model.key == "Humidity") {
                item.humidity = `${action.payload.model.value} ${action.payload.model.unit}`
              }
              if (action.payload.model.key == "Temperature") {
                item.temperature = `${action.payload.model.value} ${action.payload.model.unit}`
              }
            }
          })
        }
      }

      // alert(JSON.stringify(action.payload.model))


      console.log(state)
      let val = '';
      // switch( action.payload.model.value ) {
      //   case '开':
      //     val = 'on';
      //     break;
      //   case '关':
      //     val = 'off';
      //     break;
      //   default:
      //     '';
      // }
      return {
        ...state,
        status: val,
      };
    },
  },

  subscriptions: {
    setup({ dispatch, history }): void {
      emitter.on('message', function (payload) {

        /**
         * deviceId: "75bae534-a2ff-4355-97cf-06e7acbacc57"
         * id: "f9f71b12-134e-4b18-ba33-335620ea6bc5"
         */
        if (payload.event === "dataEntityUpdated") {
          //

          if (dispatch) {
            dispatch({
              type: 'update',
              payload
            });
          }
          //console.log('message::::', payload)
        }


      })
    },
  },
};

export default Model;
